### -*-python-*-
###
### GNU Affero General Public License compliance
###
### (c) 2013 Mark Wooding
###

###----- Licensing notice ---------------------------------------------------
###
### This file is part of Chopwood: a password-changing service.
###
### Chopwood is free software; you can redistribute it and/or modify
### it under the terms of the GNU Affero General Public License as
### published by the Free Software Foundation; either version 3 of the
### License, or (at your option) any later version.
###
### Chopwood is distributed in the hope that it will be useful,
### but WITHOUT ANY WARRANTY; without even the implied warranty of
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
### GNU Affero General Public License for more details.
###
### You should have received a copy of the GNU Affero General Public
### License along with Chopwood; if not, see
### <http://www.gnu.org/licenses/>.

import contextlib as CTX
import os as OS
import shlex as SL
import shutil as SH
import subprocess as SUB
import sys as SYS
import tarfile as TAR
import tempfile as TF

from auto import PACKAGE, VERSION
import util as U

@CTX.contextmanager
def tempdir():
  d = TF.mkdtemp()
  try: yield d
  finally: SH.rmtree(d, ignore_errors = True)

def dirs_to_dump():
  dirs = set()
  for m in SYS.modules.itervalues():
    try: f = m.__file__
    except AttributeError: continue
    d = OS.path.realpath(OS.path.dirname(f))
    if d.startswith('/usr/') and not d.startswith('/usr/local/'): continue
    dirs.add(d)
  dirs = sorted(dirs)
  last = '!'
  dump = []
  for d in dirs:
    if d.startswith(last): continue
    dump.append(d)
    last = d
  return dump

def exists_subdir(subdir):
  return lambda dir: OS.path.isdir(OS.path.join(dir, subdir))

def filez(cmd):
  def _(dir):
    kid = SUB.Popen(SL.split(cmd), stdout = SUB.PIPE, cwd = dir)
    left = ''
    while True:
      buf = kid.stdout.read(16384)
      if not buf: break
      buf = left + buf
      i = 0
      while True:
        z = buf.find('\0', i)
        if z < 0: break
        f = buf[i:z]
        if f.startswith('./'): f = f[2:]
        yield f
        i = z + 1
      left = buf[i:]
    if left:
      raise U.ExpectedError, \
            (500, "trailing junk from `%s' in `%s'" % (cmd, dir))
  return _

DUMPERS = [
  (exists_subdir('.git'), [filez('git ls-files -coz --exclude-standard'),
                           filez('find .git -print0')]),
  (lambda d: True, [filez('find . ( ! -perm +004 -prune ) -o -print0')])]

def dump_dir(dir, tf, root):
  for test, listers in DUMPERS:
    if test(dir): break
  else:
    raise U.ExpectedError, (500, "no dumper for `%s'" % dir)
  for lister in listers:
    base = OS.path.basename(dir)
    for file in lister(dir):
      tf.add(OS.path.join(dir, file), OS.path.join(root, base, file),
             recursive = False)

def source(out):
  if SYS.version_info >= (2, 6):
    tf = TAR.open(fileobj = out, mode = 'w|gz', format = TAR.USTAR_FORMAT)
  else:
    tf = TAR.open(fileobj = out, mode = 'w|gz')
    tf.posix = True
  for d in dirs_to_dump():
    dump_dir(d, tf, '%s-%s' % (PACKAGE, VERSION))
  tf.close()

###----- That's all, folks --------------------------------------------------
