### -*-python-*-
###
### User commands
###
### (c) 2013 Mark Wooding
###

###----- Licensing notice ---------------------------------------------------
###
### This file is part of Chopwood: a password-changing service.
###
### Chopwood is free software; you can redistribute it and/or modify
### it under the terms of the GNU Affero General Public License as
### published by the Free Software Foundation; either version 3 of the
### License, or (at your option) any later version.
###
### Chopwood is distributed in the hope that it will be useful,
### but WITHOUT ANY WARRANTY; without even the implied warranty of
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
### GNU Affero General Public License for more details.
###
### You should have received a copy of the GNU Affero General Public
### License along with Chopwood; if not, see
### <http://www.gnu.org/licenses/>.

from __future__ import with_statement

import getpass as GP
import sys as SYS

import cmdutil as CU
import dbmaint as D
import operation as OP
from output import PRINT
import service as S
import subcommand as SC
import util as U

OMSG = [None,
        "Partial failure",
        "Operation failed",
        "No services selected"]

def operate(op, accts, *args, **kw):
  """
  Perform a request as indicated by the arguments (see `operation.operate'
  for the full details), and report the results.
  """

  ## Collect the results.
  o, ii, rq, ops = OP.operate(op, accts, *args, **kw)

  ## Report any additional information collected.
  if o.nwin and ii:
    CU.format_list(ii,
                   [CU.column('RESULT', "~={0.desc}A"),
                    CU.column('VALUE', "~={0.value}A")])
    PRINT()

  ## Report the outcomes of the indvidual operations.
  if ops:
    CU.format_list(ops,
                   [CU.column('SERVICE',
                              "~={0.svc.friendly}A"),
                    CU.column('RESULT', "~={0.error}:["
                              "OK~={0.result}@[ ~A~]~;"
                              "FAILED: ~={0.error.msg}A~]")])

  ## If it failed, report an appropriate error.
  if o.rc:
    if o.nlose: PRINT()
    raise U.ExpectedError, (400, OMSG[o.rc])

@SC.subcommand(
  'set', ['userv'],
  """Sets the password for the SERVICES to a given string.  If standard input
is a terminal, read the password interactively, with prompts, disabling echo,
and asking for confirmation to catch typos.  Otherwise, just read one line
and use the result as the password.""",
  rparam = SC.Arg('services'))
def cmd_set_userv(services):
  accts = CU.resolve_accounts(CU.USER, services)
  if not SYS.stdin.isatty():
    new = U.readline('new password')
  else:
    first = GP.getpass('Enter new password: ')
    second = GP.getpass('Confirm new password: ')
    if first != second: raise U.ExpectedError, (400, "Passwords don't match")
    new = first
  operate('set', accts, new)

@SC.subcommand(
  'reset', ['userv'],
  """Resets the password for the SERVICES.""",
  rparam = SC.Arg('services'))
def cmd_reset_userv(services):
  accts = CU.resolve_accounts(CU.USER, services)
  operate('reset', accts)

@SC.subcommand(
  'clear', ['userv'],
  """Clears the password for the SERVICES, preventing access.  This doesn't
work for all services, depending on how passwords are represented.""",
  rparam = SC.Arg('services'))
def cmd_clear_userv(services):
  accts = CU.resolve_accounts(CU.USER, services)
  operate('clear', accts)

@SC.subcommand('list', ['userv'], 'List available accounts')
def cmd_list_userv():
  CU.format_list(CU.list_accounts(CU.USER),
                 [CU.column('NAME', "~={0.service}A"),
                  CU.column('DESCRIPTION', "~={0.friendly}A"),
                  CU.column('LOGIN', "~={0.alias}:[---~;~={0.alias}A~]")])

###----- That's all, folks --------------------------------------------------
